# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: zip_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11zip_service.proto\x12\nzipservice\"\x19\n\nZipRequest\x12\x0b\n\x03\x64ir\x18\x01 \x01(\t\"\x1f\n\x0bZipProgress\x12\x10\n\x08progress\x18\x01 \x01(\x02\"#\n\x0f\x44ownloadRequest\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\"?\n\x08ZipChunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x12\n\ntotal_size\x18\x02 \x01(\x03\x12\x11\n\tsent_size\x18\x03 \x01(\x03\x32\x8f\x01\n\nZipService\x12=\n\x08StartZip\x12\x16.zipservice.ZipRequest\x1a\x17.zipservice.ZipProgress0\x01\x12\x42\n\x0b\x44ownloadZip\x12\x1b.zipservice.DownloadRequest\x1a\x14.zipservice.ZipChunk0\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zip_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ZIPREQUEST']._serialized_start=33
  _globals['_ZIPREQUEST']._serialized_end=58
  _globals['_ZIPPROGRESS']._serialized_start=60
  _globals['_ZIPPROGRESS']._serialized_end=91
  _globals['_DOWNLOADREQUEST']._serialized_start=93
  _globals['_DOWNLOADREQUEST']._serialized_end=128
  _globals['_ZIPCHUNK']._serialized_start=130
  _globals['_ZIPCHUNK']._serialized_end=193
  _globals['_ZIPSERVICE']._serialized_start=196
  _globals['_ZIPSERVICE']._serialized_end=339
# @@protoc_insertion_point(module_scope)
